==> rtai-0.5.tgz: rtai-0.5/mups_examples/sw/rt_process.c <==
/* Produce a rectangular wave on output 0 of a parallel port */

#define MODULE
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>

#include <asm/system.h>
#include <asm/i82489.h>
#include <asm/fixmap.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>

#include <asm/system.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>
#include <asm/io.h>

#include <rtai.h>
#include <rtai_mups.h>

/* the address of the parallel port -- you probably should change this */

#define LPT 0x378

#define TICK_PERIOD 20000

#define STACK_SIZE 2000

#define LOOPS 100000000

#define NTASKS 16

RT_TASK thread[NTASKS];

static int cpu_used[NR_RT_CPUS];

void fun(int t)
{
	unsigned int loops = LOOPS;
	while(loops--) {
		cpu_used[hard_cpu_id()]++;
		outb(t, LPT);
		rt_task_wait_period();
	}
}

static struct apic_timer_setup_data apic_setup_data[NR_RT_CPUS] = 
			{ {0, TICK_PERIOD}, {1, TICK_PERIOD} };

int init_module(void)
{
	RTIME now;
	int i, k;

	k = 1;
	for (i = 0; i < NTASKS; i++) {
		rt_task_init(&thread[i], fun, k = ~k, STACK_SIZE, 0, 0, 0, i%2);
	}
	start_rt_timer(apic_setup_data, 0);
	now = rt_get_time_ns() + 100000000;
	for (i = 0; i < NTASKS; i++) {
		rt_task_make_periodic(&thread[i], now + (i+1)*TICK_PERIOD, NTASKS*TICK_PERIOD);
	}
	return 0;
}


void cleanup_module(void)
{
	int i, cpuid;

	rt_reset_irq_to_sym_mode(TIMER_8254_IRQ);
	stop_rt_timer();
	rt_busy_sleep(1E7);
	for (i = 0; i < NTASKS; i++) {
		rt_task_delete(&thread[i]);
	}
	printk("\n\nCPU USE SUMMARY\n");
	for (cpuid = 0; cpuid < NR_RT_CPUS; cpuid++) {
		printk("# %d -> %d\n", cpuid, cpu_used[cpuid]);
	}
	printk("END OF CPU USE SUMMARY\n\n");
}

==> rtai-0.9a.tgz: rtai-0.9a/mups_examples/sw/rt_process.c <==
/* Produce a rectangular wave on output 0 of a parallel port */

#define MODULE
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>

#include <asm/system.h>
#include <asm/i82489.h>
#include <asm/fixmap.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>

#include <asm/system.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>
#include <asm/io.h>

#include <rtai.h>
#include <rtai_sched.h>

/* the address of the parallel port -- you probably should change this */

#define LPT 0x378

#define TICK_PERIOD 20000

#define STACK_SIZE 2000

#define LOOPS 100000000

#define NTASKS 16

RT_TASK thread[NTASKS];

static int cpu_used[NR_RT_CPUS];

void fun(int t)
{
	unsigned int loops = LOOPS;
	while(loops--) {
		cpu_used[hard_cpu_id()]++;
		outb(t, LPT);
		rt_task_wait_period();
	}
}

static struct apic_timer_setup_data apic_setup_data[NR_RT_CPUS] = 
			{ {1, TICK_PERIOD}, {0, TICK_PERIOD} };

int init_module(void)
{
	int i, k;

	k = 1;
	for (i = 0; i < NTASKS; i++) {
		rt_task_init_cpuid(&thread[i], fun, k = ~k, STACK_SIZE, 0, 0, 0, i%2 + 1);
	}
	start_rt_apic_timers(apic_setup_data, 0);
	for (i = 0; i < NTASKS; i++) {
		rt_task_make_periodic_relative_ns(&thread[i], (i+1)*TICK_PERIOD, NTASKS*TICK_PERIOD);
	}
	return 0;
}


void cleanup_module(void)
{
	int i, cpuid;

	stop_rt_timer();
	for (i = 0; i < NTASKS; i++) {
		rt_task_delete(&thread[i]);
	}
	printk("\n\nCPU USE SUMMARY\n");
	for (cpuid = 0; cpuid < NR_RT_CPUS; cpuid++) {
		printk("# %d -> %d\n", cpuid, cpu_used[cpuid]);
	}
	printk("END OF CPU USE SUMMARY\n\n");
}
